@file:OptIn(ExperimentalMaterial3Api::class)

package com.example.app.screens

import android.content.Context
import android.widget.Toast
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.LinearProgressIndicator
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import com.arcgismaps.Color
import com.arcgismaps.geometry.Envelope
import com.arcgismaps.geometry.SpatialReference
import com.arcgismaps.mapping.ArcGISMap
import com.arcgismaps.mapping.PortalItem
import com.arcgismaps.mapping.symbology.SimpleFillSymbol
import com.arcgismaps.mapping.symbology.SimpleFillSymbolStyle
import com.arcgismaps.mapping.symbology.SimpleLineSymbol
import com.arcgismaps.mapping.symbology.SimpleLineSymbolStyle
import com.arcgismaps.mapping.view.Graphic
import com.arcgismaps.mapping.view.GraphicsOverlay
import com.arcgismaps.portal.Portal
import com.arcgismaps.tasks.offlinemaptask.GenerateOfflineMapParameters
import com.arcgismaps.tasks.offlinemaptask.OfflineMapTask
import com.arcgismaps.toolkit.geoviewcompose.MapView
import com.example.app.R
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.launch
import java.util.Calendar

@Composable
fun MainScreen() {

    val context = LocalContext.current
    val coroutineScope = rememberCoroutineScope()

    val currentProgress = remember { mutableIntStateOf(0) }
    val showProgressBar = remember { mutableStateOf(false) }

    val graphicsOverlay = remember { GraphicsOverlay() }
    val graphicsOverlays = remember { listOf(graphicsOverlay) }

    val offlineArea = remember {
        createAreaOfInterest(
            onOfflineAreaCreated = { graphic -> graphicsOverlay.graphics.add(graphic) }
        )
    }

    var map = remember {
        createMap()
    }

    LaunchedEffect(Unit) {

        // Create an offline map task using the current map
        val offlineMapTask = OfflineMapTask(map)

        // Create a default set of parameters for generating the offline map from the area of interest
        offlineMapTask.createDefaultGenerateOfflineMapParameters(offlineArea)
            .onSuccess { parameters ->

                // When the parameters are successfully created, show progress bar and message
                showProgressBar.value = true
                showMessage(context, "Default parameters have been created.")

                // Download the offline map
                downloadOfflineMapArea(
                    context = context,
                    coroutineScope = coroutineScope,
                    offlineMapTask = offlineMapTask,
                    parameters = parameters,
                    updateProgress = { currentProgress.intValue = it },
                    onDownloadOfflineMapSuccess = { offlineMap ->
                        // Hide progress bar and show message when the job executes successfully
                        showProgressBar.value = false
                        showMessage(context, "Downloaded offline map area successfully.")
                        // Replace the current map with the result offline map
                        map = offlineMap
                        // Clear the offline area outline
                        graphicsOverlay.graphics.clear()
                    },
                    onDownloadOfflineMapFailure = {
                        showProgressBar.value = false
                        showMessage(context, "Failed to download offline map area.")
                    }
                )

            }

    }

    Scaffold(
        topBar = { TopAppBar(title = { Text(text = stringResource(id = R.string.app_name)) }) }
    ) {

        Column(
            modifier = Modifier.fillMaxSize().padding(it)
        ) {
            if (showProgressBar.value) {
                LinearProgressIndicator(
                    modifier = Modifier.fillMaxWidth(),
                    progress = currentProgress.intValue / 100f
                )
            }

            MapView(
                modifier = Modifier.fillMaxSize(),
                arcGISMap = map,

                graphicsOverlays = graphicsOverlays

            )

        }

    }

}

fun createMap(): ArcGISMap {

    val portal = Portal(
        url = "https://www.arcgis.com",
        connection = Portal.Connection.Anonymous
    )
    val portalItem = PortalItem(
        portal = portal,
        itemId = "acc027394bc84c2fb04d1ed317aac674"
    )
    return ArcGISMap(portalItem)

}

fun createAreaOfInterest(
    onOfflineAreaCreated: (Graphic) -> Unit
): Envelope {

    // Create an envelope that defines the area to take offline
    val offlineArea = Envelope(
        xMin = -88.1526,
        yMin = 41.7694,
        xMax = -88.1490,
        yMax = 41.7714,
        spatialReference = SpatialReference.wgs84()
    )

    // Create a graphic to display the area to take offline
    val lineSymbol = SimpleLineSymbol(
        style = SimpleLineSymbolStyle.Solid,
        color = Color.red,
        width = 2f
    )
    val fillSymbol = SimpleFillSymbol(
        style = SimpleFillSymbolStyle.Solid,
        color = Color.transparent,
        outline = lineSymbol
    )
    val offlineAreaGraphic = Graphic(
        geometry = offlineArea,
        symbol = fillSymbol
    )

    // Add the offline area outline to the graphics overlay
    onOfflineAreaCreated(offlineAreaGraphic)


    return offlineArea
}

suspend fun downloadOfflineMapArea(
    context: Context,
    coroutineScope: CoroutineScope,
    offlineMapTask: OfflineMapTask,
    parameters: GenerateOfflineMapParameters,
    updateProgress: (Int) -> Unit,
    onDownloadOfflineMapSuccess: (ArcGISMap) -> Unit,
    onDownloadOfflineMapFailure: () -> Unit
) {

    // Build a folder path named with today's date/time to store the offline map
    val downloadLocation =
        context.getExternalFilesDir(null)?.path + "OfflineMap_" + Calendar.getInstance().time

    // Create a job with GenerateOfflineMapParameters and a download directory path
    val generateOfflineMapJob = offlineMapTask.createGenerateOfflineMapJob(
        parameters = parameters,
        downloadDirectoryPath = downloadLocation
    )

    coroutineScope.launch {
        generateOfflineMapJob.progress.collect { progress ->
            updateProgress(progress)
        }
    }

    // Start the job to download the offline map
    generateOfflineMapJob.start()
    showMessage( context, "Generate offline map job has started.")

    generateOfflineMapJob.result().onSuccess { generateOfflineMapResult ->
        onDownloadOfflineMapSuccess(generateOfflineMapResult.offlineMap)
    }.onFailure {
        onDownloadOfflineMapFailure()
    }

}

fun showMessage(context: Context, message: String) {
    Toast.makeText(context, message, Toast.LENGTH_LONG).show()
}



